TITLE	res13.com
SUBTTL	Resident rot13 decoder
VERSION	EQU "1.00"
RELEASE_DATE EQU "04/18/93"
COMMENT	|

	Memory resident program rot13's your screen at the touch of a button.
	Press <Scroll-Lock> to decode screen. Releasing it restores your
	screen to the way it was originally.

REQUIREMENTS:	
	Must have 80286 processor or better.

	Supports color or mono text modes only.

	Supports only 25 row/80 column text mode screen. Patching instructions
	included for other than 25/80 text.

	Only supports video page 0. Shouldn't be a problem, but if it is,
	you're on your own (sorry).

SYNTAX:	 rot13 [y]
OPTIONS: 'y' - skips warning message about needing 286 or better
RETURNS: nothing

AUTHOR:	 Duane Paulson <duanep@holonet.net>
HISTORY:
	1.00 - 04/18/93 - Initial Release

|
	.286
text	SEGMENT	'CODE'
	ASSUME	CS:text
	ORG	80h
ccount	LABEL	BYTE				; count of command line chars
	ORG	81h
ctail	LABEL	BYTE				; start of command line params
	ORG	100h
 ; Entry point when program is started from command line.
start	PROC	NEAR
	jmp	SHORT	install			; jump to installation routine
start	ENDP

 ; Data needed by memory-resident process.
active		DB	0
old_isr1C	LABEL	DWORD
old_off		DW	?
old_seg		DW	?
video		DW	0B800h

 ; Code for memory-resident process begins here
isr1C	PROC	FAR
	pusha
	push	ds
	push	es

	push	cs				; make local data accessible
	pop	ds
	mov	ax,0040h			; load bios data segment to es
	mov	es,ax
	mov	ah,BYTE PTR es:[0018h]		; look at kb flag byte #1
	test	ah,00010000b			; scroll lock key depressed?
	jz	no_scr				; jump if not

 ; Scroll Lock key is depressed
	cmp	active,1			; are we already active?
	je	bye				; quit if yes
	mov	active,1			; else set active flag,
	CALL	rot13				;  and rot13 the screen
	jmp	SHORT bye

 ; Scroll Lock key is not pressed
no_scr:	cmp	active,1			; is rot13 active?
	jne	bye				; quit if not active
	mov	active,0			; if active, clear active flag,
	CALL	rot13				;  and re-rot the screen

bye:	pop	es
	pop	ds
	popa
	jmp	old_isr1C			; continue with old isr
isr1C	ENDP

rot13	PROC	NEAR
 ; called by isr to rot-13 the screen
	mov	ax,video			; load video segment to es
	mov	es,ax
	mov	bx,0				; starting offset is 0
	mov	cx,2000				; 2000 char/attrib words
 ; rot-13 processing loop		
r13l:	mov	ax,WORD PTR es:[bx]		; grab a char/attrib word
	mov	dx,ax				; and store it in original form
	or	al,32				; set the uppercase bit

	cmp	al,'a'				; are we below the alphas?
	jb	write           		; if yes, do not rot13
	cmp	al,'z'				; are we above?
	ja	write				; likewise...

	cmp	al,'n'				; ready to rotate. upper or
	jb	rot1				;  lower half of alphabet?
	sub	dl,13				; subtract 13 if upper half,
	jmp	SHORT	write
rot1:	add	dl,13				; add 13 if lower half.

write:	mov	WORD PTR es:[bx],dx		; write it back to video
	inc	bx				; increment for word value
	inc	bx				;  (2 bytes)
	loop	r13l
	ret
rot13	ENDP
 ; End of the resident code.

install	PROC	NEAR
	ASSUME	DS:text
 ; Check for command line params
	mov	ch,0
	mov	cl,ccount
	cmp	ccount,0
	je	warn
	mov	bx,0
pchk1:	mov	ah,ctail[bx]
	cmp	ah,32
	je	pchk2
	cmp	ah,9
	je	pchk2
	or	ah,32
	cmp	ah,'y'
	je	no_wrn
	jne	help
pchk2:	inc	bx
	loop	pchk1

 ; display warning message and ask if safe to continue
warn:	mov	ah,9
	mov	dx,OFFSET warning
	int	21h

	mov	ah,0				; wait for keystroke & read
	int	16h
	or	al,32
	cmp	al,'y'
	je	no_wrn

 ; Abort installation
abo:	mov	ah,9				; display abort message
	mov	dx,OFFSET abo_msg
	int	21h
	mov	ax,4C00h			; release all memory and exit
	int	21h

 ; display help message, then exit
help:	mov	ah,9
	mov	dx,OFFSET help_msg
	int	21h
	jmp	SHORT	abo	
	
 ; Proceed with installation
 ; Check for mono video
no_wrn:	mov	ah,0Fh
	int	10h
	cmp	al,7				; mode 7 is the only mono mode
	jne	inst1				; anything else is color card
	mov	video,0B000h			; mono screen segment specified
 ; Get interrupt vector for Interrupt Service Routine 1Ch
inst1:	mov	ax,351Ch
	int	21h
	mov	old_off,bx			; and store it
	mov	old_seg,es
 ; Set interrupt vector for 1Ch to our ISR
	mov	ax,251Ch
	mov	dx,OFFSET isr1C
	int	21h
 ; Display message
	mov	ah,9
	mov	dx,OFFSET imsg
	int	21h
 ; Terminate process and remain resident
	mov	ax,OFFSET install		; bytes in resident code
	mov	cl,4				; divide by 16 for paragraphs
	shr	ax,cl
	inc	ax				; include last paragraph
	mov	dx,ax
	mov	ax,3100h			; let dos know what's happening
	int	21h				; exit
install	ENDP
 ; Data used by the installation module that does not need to be preserved:
imsg	DB	""
	DB	"ķ",13,10
	DB	" Res13 ver ",VERSION
	DB	" (",RELEASE_DATE,") loaded. "
	DB	"Press <Scroll Lock> to activate. ",13,10
	DB	" Erf13 ire ",VERSION
	DB	" (",RELEASE_DATE,") ybnqrq. "
	DB	"Cerff <Fpebyy Ybpx> gb npgvingr. ",13,10
	DB	""
	DB	"Ľ",13,10,"$"

help_msg DB	"RES13.COM Short help -- "
	DB	"Use the following syntax to get started:",13,10
	DB	"     res13",13,10,"$"

abo_msg	DB	13,10,"RES13 Installation Aborted",13,10,"$"

warning	DB	""
	DB	"ķ",13,10
	DB	" RES13.COM                           "
	DB	"                                      ",13,10
	DB	"                                     "
	DB	"                                      ",13,10
	DB	" Res13 requires an 80286 processor or"
	DB	" better. Do not attempt to use res13  ",13,10
	DB	" if you have an 8088 or 8086 (Origina"
	DB	"l PC or XT). An AT (286) or later     ",13,10
	DB	" model is needed. Res13 does not test"
	DB	" for the type of machine you have.    ",13,10
	DB	"                                     "
	DB	"                                      ",13,10
	DB	" If you have a 286 or later machine, "
	DB	"proceed with the installation by      ",13,10
	DB	" pressing 'Y'  Pressing any other key"
	DB	" will cause res13 to stop without     ",13,10
	DB	" completing the installation.        "
	DB	"                                      ",13,10
	DB	"                                     "
	DB	"                                      ",13,10
	DB	" If you are sure you have the proper "
	DB	"computer, you can skip this warning   ",13,10
	DB	" by using the following command param"
	DB	"eter                                  ",13,10
	DB	"                                     "
	DB	"                                      ",13,10
	DB	"          res13 y                    "
	DB	"                                      ",13,10
	DB	"                                     "
	DB	"                                      ",13,10
	DB	" This will cause Res13 to complete in"
	DB	"stallation without pausing for this   ",13,10
	DB	" screen.                             "
	DB	"                                      ",13,10
	DB	""
	DB	"Ľ",13,10,"$"

text	ENDS
	END	start
